'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var cache$1 = require('@presta/cache');

var _id = 0;
var _reqs = [];
var _cache = {};

function isPending() {
  return !!_reqs.length;
}

function resolve() {
  var requests = Object.keys(_cache).map(function (key) {
    return _cache[key];
  }); // @ts-ignore

  return Promise.allSettled(requests);
}

function cache(key, loader) {
  if (_cache[key] !== undefined) return _cache[key];

  _reqs.push(key);

  _cache[key] = loader();

  _cache[key].then(function (res) {
    _cache[key] = res;

    _reqs.splice(_reqs.indexOf(key), 1);
  });

  return null;
}

function load(loader, options) {
  if (options === void 0) {
    options = {};
  }

  var _a = options.key,
      key = _a === void 0 ? _id++ + "" : _a,
      duration = options.cache;

  if (key && duration) {
    return cache(key, function () {
      return cache$1.cache(loader, {
        key: key,
        duration: duration
      });
    });
  }

  return cache(key, loader);
}
function render(component, ctx, renderer) {
  return tslib.__awaiter(this, void 0, void 0, function () {
    var body;
    return tslib.__generator(this, function (_a) {
      switch (_a.label) {
        case 0:
          body = renderer(component, ctx);
          if (!isPending()) return [3
          /*break*/
          , 2];
          return [4
          /*yield*/
          , resolve()];

        case 1:
          _a.sent(); // reset on each render


          _id = 0;
          return [2
          /*return*/
          , render(component, ctx, renderer)];

        case 2:
          if (_reqs.length) {
            throw new Error("@presta/load - unresolved requests: " + JSON.stringify(_reqs));
          }

          return [2
          /*return*/
          , tslib.__assign(tslib.__assign({}, ctx), {
            body: body + (ctx.body || ''),
            data: tslib.__assign(tslib.__assign({}, _cache), ctx.data || {})
          })];
      }
    });
  });
}

exports.load = load;
exports.render = render;
